import React, { useEffect, useMemo, useCallback, useRef } from 'react';
import { connect, history } from 'umi';
import { PageContainer } from '@ant-design/pro-layout';
import ProTable from '@ant-design/pro-table';
import { PlusOutlined } from '@ant-design/icons';
import { Button, notification } from 'antd';
import request, { queryTable } from '@/utils/request';

const PostList = (props) => {
  const tableRef = useRef()

  const handlePostClick = useCallback((post) => {
    history.push(post ? `/post/edit?id=${post.postId}` : '/post/edit');
  }, []);

  const emitPublishOrNot = (post) => {
    // 1 变成 0, 0 变成 1
    const status = Math.abs(post.status - 1)
    request(`/post/${post.postId}`, { method: 'put', data: { ...post, status } }).then(() => {
      notification.success({ message: '操作成功' })
      tableRef.current.reload()
    }).catch((e) => {
      notification.warn({ message: e.message })
    })
  }

  const actions = [
    <Button key="button" icon={<PlusOutlined />} type="primary" onClick={() => handlePostClick()}>
      新建
    </Button>,
  ];

  const typeDict = useMemo(() => {
    return (props.typeList || []).reduce((acc, item) => {
      return {
        ...acc,
        [item.typeId]: { text: item.name, status: 'Default' },
      };
    }, {});
  }, [props.typeList]);

  const columns = [
    {
      title: '标题',
      key: 'name',
      dataIndex: 'name',
      hideInTable: true,
    },
    {
      title: '分类',
      key: 'typeId',
      dataIndex: 'typeId',
      hideInTable: true,
      valueType: 'select',
      valueEnum: typeDict,
    },
    {
      title: '海报',
      key: 'poster',
      dataIndex: 'poster',
      width: 100,
      hideInSearch: true,
      render: (url) => <img width={64} src={url} alt="" />,
    },
    {
      title: '科普知识',
      key: 'postId',
      dataIndex: 'postId',
      hideInSearch: true,
      render: (_, row) => (
        <div>
          <h3>{row.name}</h3>
          <p>
            <small>{`作者: ${row.author}`}</small>
          </p>
          <p>
            <small>{row.isVideo ? '视频' : '图文'}</small>
          </p>
        </div>
      ),
    },
    {
      title: '分类',
      key: 'typeName',
      dataIndex: 'typeName',
      hideInSearch: true,
    },
    {
      title: '状态',
      key: 'status',
      dataIndex: 'status',
      valueType: 'select',
      valueEnum: {
        0: {
          text: '下架',
          status: 'Processing',
        },
        1: {
          text: '上架',
          status: 'Success',
        },
      },
    },
    {
      title: '创建时间',
      key: 'createDate',
      dataIndex: 'createDate',
      valueType: 'date',
      hideInSearch: true,
    },
    {
      title: '操作',
      key: 'option',
      valueType: 'option',
      render: (_, item) => [
        <a key="edit" onClick={() => history.push(`/post/edit?id=${item.postId}`)}>
          编辑
        </a>,
        <a key="up" onClick={() => emitPublishOrNot(item)}>
          {item.status === 1 ? '下架' : '上架'}
        </a>,
      ],
    },
  ];

  useEffect(() => {
    if (!props.typeList || !props.typeList.length) {
      props.dispatch({
        type: 'post/getTypeList',
        payload: { pageSize: 999 },
      });
    }
  }, [props]);

  return (
    <PageContainer>
      <ProTable
        actionRef={tableRef}
        columns={columns}
        request={queryTable('/post')}
        rowKey="postId"
        headerTitle="科普列表"
        search={{
          labelWidth: '4em',
        }}
        toolBarRender={() => actions}
      />
    </PageContainer>
  );
};

export default connect((s) => ({
  typeList: s.post.typeList,
}))(PostList);
